@@ -336,6 +336,11 @@ eel_gnome_canvas_request_update_all (GnomeCanvas *canvas)
  * even though it should. The update is in there with an #if 0 around
  * it, with no explanation of why it's commented out. For now, work
  * around this by requesting an update explicitly.
+ * ALEX: The update was causing us to repaint everything on every
+ * relayout. An object may have data stored that depends on x1 and y1,
+ * since they decide the coordinate space transform. Nothing should
+ * depend on x2 and y2 though. This lets us grow the canvas to the
+ * right and down without performance problems.
  */
 void
 eel_gnome_canvas_set_scroll_region (GnomeCanvas *canvas,
@@ -348,7 +353,9 @@ eel_gnome_canvas_set_scroll_region (GnomeCanvas *canvas,
 	gnome_canvas_get_scroll_region (canvas, &old_x1, &old_y1, &old_x2, &old_y2);
 	if (old_x1 != x1 || old_y1 != y1 || old_x2 != x2 || old_y2 != y2) {
 		gnome_canvas_set_scroll_region (canvas, x1, y1, x2, y2);
-		eel_gnome_canvas_request_update_all (canvas);
+		if (old_x1 != x1 || old_y1 != y1) {
+			eel_gnome_canvas_request_update_all (canvas);
+		}
 		gnome_canvas_item_request_update (canvas->root);
 	}
 }
@@ -406,102 +413,14 @@ eel_gnome_canvas_set_scroll_region_include_visible_area (GnomeCanvas *canvas,
 }
 
 
-/* Code from GMC, contains all the voodoo needed to start
- * a terminal from the file manager nicely
- */
-
-static int
-max_open_files (void)
-{
-	static int files;
-
-	if (files != 0) {
-		return files;
-	}
-
-#ifdef HAVE_SYSCONF
-	files = sysconf (_SC_OPEN_MAX);
-	if (files != -1) {
-		return files;
-	}
-#endif
-#ifdef OPEN_MAX
-	files = OPEN_MAX;
-#else
-	files = 256;
-#endif
-	return files;
-}
-
-int 
+void 
 eel_gnome_shell_execute (const char *command)
 {
-	struct sigaction ignore, save_intr, save_quit, save_stop;
-	int status, i;
-	int pid;
-	
-	ignore.sa_handler = SIG_IGN;
-	sigemptyset (&ignore.sa_mask);
-	ignore.sa_flags = 0;
-	status = 0;
-    
-	sigaction (SIGINT, &ignore, &save_intr);    
-	sigaction (SIGQUIT, &ignore, &save_quit);
-
-	pid = fork ();
-	if (pid < 0){
-		return -1;
+	GError *error = NULL;
+	if (!g_spawn_command_line_async (command, &error)) {
+		g_warning ("Error starting command '%s': %s\n", command, error->message);
+		g_error_free (error);
 	}
-	
-	if (pid == 0){
-		int top;
-		struct sigaction default_pipe;
-
-		top = max_open_files ();
-		sigaction (SIGINT,  &save_intr, NULL);
-		sigaction (SIGQUIT, &save_quit, NULL);
-
-		/*
-		 * reset sigpipe
-		 */
-		default_pipe.sa_handler = SIG_DFL;
-		sigemptyset (&default_pipe.sa_mask);
-		default_pipe.sa_flags = 0;
-		
-		sigaction (SIGPIPE, &default_pipe, NULL);
-		
-		for (i = 0; i < top; i++)
-			close (i);
-
-		/* Setup the file descriptor for the child */
-		   
-		/* stdin */
-		open ("/dev/null", O_APPEND);
-
-		/* stdout */
-		open ("/dev/null", O_RDONLY);
-
-		/* stderr */
-		open ("/dev/null", O_RDONLY);
-		
-		pid = fork ();
-		if (pid == 0){
-			execl ("/bin/sh", "/bin/sh", "-c", command, (char *) 0);
-			/* See note below for why we use _exit () */
-			_exit (127);		/* Exec error */
-		}
-		/* We need to use _exit instead of exit to avoid
-		 * calling the atexit handlers (specifically the gdk atexit
-		 * handler
-		 */
-		_exit (0);
-	}
-	waitpid (pid, &status, 0);
-	sigaction (SIGINT,  &save_intr, NULL);
-	sigaction (SIGQUIT, &save_quit, NULL);
-	sigaction (SIGTSTP, &save_stop, NULL);
-
-	return WEXITSTATUS(status);
 }
 
 /* Return a command string containing the path to a terminal on this system. */
