@@ -80,12 +80,17 @@
 void
 eel_gtk_button_set_padding (GtkButton *button, int pad_amount)
 {
+	GtkWidget *child;
+
 	g_return_if_fail (GTK_IS_BUTTON (button));
 	g_return_if_fail (pad_amount > 0);
 
-	gtk_misc_set_padding (GTK_MISC (GTK_BIN(button)->child), 
-			      pad_amount, 
-			      pad_amount);
+	child = GTK_BIN (button)->child;
+
+	if (GTK_IS_MISC (child)) {
+		gtk_misc_set_padding
+			(GTK_MISC (child), pad_amount, pad_amount);
+	}
 }
 
 /**
@@ -497,9 +502,14 @@ eel_pop_up_context_menu (GtkMenu	     *menu,
 	 * other than the one that invoked the menu will be ignored (instead
 	 * of dismissing the menu). This is a subtle fragility of the GTK menu code.
 	 */
-	button = event->type == GDK_BUTTON_RELEASE
-		? 0
-		: event->button;
+
+	if (event) {
+		button = event->type == GDK_BUTTON_RELEASE
+			? 0
+			: event->button;
+	} else {
+		button = 0;
+	}
 	
 	gtk_menu_popup (menu,					/* menu */
 			NULL,					/* parent_menu_shell */
@@ -507,7 +517,7 @@ eel_pop_up_context_menu (GtkMenu	     *menu,
 			eel_popup_menu_position_func,	/* func */
 			&offset,			        /* data */
 			button,					/* button */
-			event->time);				/* activate_time */
+			event ? event->time : GDK_CURRENT_TIME); /* activate_time */
 
 	gtk_object_sink (GTK_OBJECT (menu));
 }
@@ -1079,3 +1089,86 @@ eel_gtk_tree_view_cell_is_completely_visible (GtkTreeView          *tree_view,
 	gtk_tree_view_get_visible_rect (tree_view, &visible_rect);
 	return eel_gdk_rectangle_contains_rectangle (visible_rect, cell_rect);
 }
+
+gboolean
+eel_gtk_viewport_get_visible_rect (GtkViewport  *viewport, 
+				   GdkRectangle *rect)
+{
+	GdkRectangle viewport_rect;
+	GdkRectangle child_rect;
+	gboolean return_val;
+
+	g_return_val_if_fail (GTK_IS_VIEWPORT (viewport), FALSE);
+	g_return_val_if_fail (rect != NULL, FALSE);
+	
+	if (GTK_WIDGET_REALIZED (viewport)) {
+		viewport_rect.x = 0;
+		viewport_rect.y = 0;
+		gdk_drawable_get_size (viewport->view_window, 
+				       &viewport_rect.width, 
+				       &viewport_rect.height);
+		
+		gdk_window_get_position (viewport->bin_window,
+					 &child_rect.x,
+					 &child_rect.y);
+		gdk_drawable_get_size (viewport->bin_window,
+				       &child_rect.width,
+				       &child_rect.height);
+
+		return_val = gdk_rectangle_intersect (&viewport_rect, 
+						      &child_rect,
+						      rect);
+		rect->x -= child_rect.x;
+		rect->y -= child_rect.y;
+		
+		return return_val;
+	}
+	
+	rect->x = rect->y = rect->width = rect->height = 0;
+	return FALSE;
+}
+
+void
+eel_gtk_viewport_scroll_to_rect (GtkViewport  *viewport, 
+				 GdkRectangle *rect)
+{
+	GdkRectangle visible_rect;
+	int scroll_x;
+	int scroll_y;
+	GtkAdjustment *adjustment;
+
+	g_return_if_fail (GTK_IS_VIEWPORT (viewport));
+	g_return_if_fail (rect != NULL);
+
+	if (eel_gtk_viewport_get_visible_rect (viewport, &visible_rect)) {
+		scroll_x = -1;
+		scroll_y = -1;
+
+		if (rect->x + rect->width > visible_rect.x + visible_rect.width) {
+			scroll_x = rect->x - (visible_rect.width - rect->width);
+		}
+		if (rect->y + rect->height > visible_rect.y + visible_rect.height) {
+			scroll_y = rect->y - (visible_rect.height - rect->height);
+		}
+
+		if (rect->x < visible_rect.x) {
+			scroll_x = rect->x;
+		}
+
+		if (rect->y < visible_rect.y) {
+			scroll_y = rect->y;
+		}
+
+		adjustment = gtk_viewport_get_hadjustment (viewport);
+		if (adjustment && scroll_x != -1) {
+			eel_gtk_adjustment_set_value (adjustment,
+						      (double)scroll_x);
+		}
+
+		adjustment = gtk_viewport_get_vadjustment (viewport);
+		if (adjustment && scroll_y != -1) {
+			eel_gtk_adjustment_set_value (adjustment,
+						      (double)scroll_y);
+		}
+	}
+}
